package sun.awt;

import java.awt.Graphics;
import java.awt.Component;
import java.awt.Rectangle;

public class RepaintArea
{
    private static final int MAX_BENEFIT_RATIO = 4;
    private static final int HORIZONTAL = 0;
    private static final int VERTICAL = 1;
    private static final int UPDATE = 2;
    private static final int RECT_COUNT = 3;
    private Rectangle[] paintRects;
    
    public RepaintArea() {
        this.paintRects = new Rectangle[3];
    }
    
    private RepaintArea(RepaintArea repaintArea) {
        this.paintRects = new Rectangle[3];
        for (int i = 0; i < 3; ++i) {
            this.paintRects[i] = repaintArea.paintRects[i];
        }
    }
    
    public synchronized void add(Rectangle rectangle, int n) {
        if (rectangle.isEmpty()) {
            return;
        }
        int n2 = 2;
        if (n == 800) {
            n2 = ((rectangle.width > rectangle.height) ? 0 : 1);
        }
        if (this.paintRects[n2] != null) {
            this.paintRects[n2].add(rectangle);
        }
        else {
            this.paintRects[n2] = new Rectangle(rectangle);
        }
        return;
    }
    
    private synchronized RepaintArea cloneAndReset() {
        RepaintArea repaintArea = new RepaintArea(this);
        for (int i = 0; i < 3; ++i) {
            this.paintRects[i] = null;
        }
        return repaintArea;
    }
    
    public boolean isEmpty() {
        int i = 0;
        while (i < 3) {
            if (this.paintRects[i] != null) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    public synchronized void constrain(int x, int y, int n, int n2) {
        for (int i = 0; i < 3; ++i) {
            Rectangle rectangle = this.paintRects[i];
            if (rectangle != null) {
                if (rectangle.x < x) {
                    rectangle.width -= x - rectangle.x;
                    rectangle.x = x;
                }
                if (rectangle.y < y) {
                    rectangle.height -= y - rectangle.y;
                    rectangle.y = y;
                }
                int n3 = rectangle.x + rectangle.width - x - n;
                if (n3 > 0) {
                    rectangle.width -= n3;
                }
                int n4 = rectangle.y + rectangle.height - y - n2;
                if (n4 > 0) {
                    rectangle.height -= n4;
                }
                if (rectangle.width <= 0 || rectangle.height <= 0) {
                    this.paintRects[i] = null;
                }
            }
        }
    }
    
    public synchronized void subtract(int x, int y, int width, int height) {
        Rectangle rectangle = new Rectangle(x, y, width, height);
        for (int i = 0; i < 3; ++i) {
            if (subtract(this.paintRects[i], rectangle) && this.paintRects[i] != null && this.paintRects[i].isEmpty()) {
                this.paintRects[i] = null;
            }
        }
    }
    
    public void paint(Object o, boolean b) {
        Component component = (Component)o;
        if (this.isEmpty()) {
            return;
        }
        if (!component.isVisible()) {
            return;
        }
        RepaintArea cloneAndReset = this.cloneAndReset();
        if (!subtract(cloneAndReset.paintRects[1], cloneAndReset.paintRects[0])) {
            subtract(cloneAndReset.paintRects[0], cloneAndReset.paintRects[1]);
        }
        if (cloneAndReset.paintRects[0] != null && cloneAndReset.paintRects[1] != null) {
            Rectangle union = cloneAndReset.paintRects[0].union(cloneAndReset.paintRects[1]);
            int n = union.width * union.height;
            if (4 * (n - cloneAndReset.paintRects[0].width * cloneAndReset.paintRects[0].height - cloneAndReset.paintRects[1].width * cloneAndReset.paintRects[1].height) < n) {
                cloneAndReset.paintRects[0] = union;
                cloneAndReset.paintRects[1] = null;
            }
        }
        for (int i = 0; i < this.paintRects.length; ++i) {
            if (cloneAndReset.paintRects[i] != null && !cloneAndReset.paintRects[i].isEmpty()) {
                Graphics graphics = component.getGraphics();
                if (graphics != null) {
                    try {
                        graphics.setClip(cloneAndReset.paintRects[i]);
                        if (i == 2) {
                            this.updateComponent(component, graphics);
                        }
                        else {
                            if (b) {
                                graphics.clearRect(cloneAndReset.paintRects[i].x, cloneAndReset.paintRects[i].y, cloneAndReset.paintRects[i].width, cloneAndReset.paintRects[i].height);
                            }
                            this.paintComponent(component, graphics);
                        }
                    }
                    finally {
                        graphics.dispose();
                    }
                }
            }
        }
        return;
    }
    
    protected void updateComponent(Component component, Graphics g) {
        if (component != null) {
            component.update(g);
        }
    }
    
    protected void paintComponent(Component component, Graphics g) {
        if (component != null) {
            component.paint(g);
        }
    }
    
    static boolean subtract(Rectangle rectangle, Rectangle r) {
        if (rectangle == null || r == null) {
            return true;
        }
        Rectangle intersection = rectangle.intersection(r);
        if (intersection.isEmpty()) {
            return true;
        }
        if (rectangle.x == intersection.x && rectangle.y == intersection.y) {
            if (rectangle.width == intersection.width) {
                rectangle.y += intersection.height;
                rectangle.height -= intersection.height;
                return true;
            }
            if (rectangle.height == intersection.height) {
                rectangle.x += intersection.width;
                rectangle.width -= intersection.width;
                return true;
            }
        }
        else if (rectangle.x + rectangle.width == intersection.x + intersection.width && rectangle.y + rectangle.height == intersection.y + intersection.height) {
            if (rectangle.width == intersection.width) {
                rectangle.height -= intersection.height;
                return true;
            }
            if (rectangle.height == intersection.height) {
                rectangle.width -= intersection.width;
                return true;
            }
        }
        return false;
    }
    
    @Override
    public String toString() {
        return super.toString() + "[ horizontal=" + this.paintRects[0] + " vertical=" + this.paintRects[1] + " update=" + this.paintRects[2] + "]";
    }
}
